"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createCfunc = void 0;
const fs_1 = __importDefault(require("fs"));
const vm_1 = require("vm");
const types_extractor_1 = require("./types-extractor");
const runtime_1 = require("./runtime");
const compiler_1 = require("./compiler");
const util_1 = require("./util");
const util_2 = require("../server/util");
const endent_1 = __importDefault(require("endent"));
const defaultTimeout = 1 * 1000 * 60; // 1min
const defaultMemoryLimit = 500 * 1024 * 1024; // 5Mb
const baseLanguageSubset = '()=>{};async()=>{};';
const languageSubset = fs_1.default.readFileSync(__dirname + '/javascript-subset.txt', 'utf-8');
const allowedNodeTypes = (0, types_extractor_1.extractTypes)(baseLanguageSubset + languageSubset);
const reserverGlobalIdentifiers = [
    'createRuntime',
    '__globals'
];
const readOnlyNatives = {
    console: (0, util_2.readOnly)((0, util_2.getConsole)(), (0, util_1.createGetTrap)([
        'log',
        'error',
        'warn',
    ])),
    Object: (0, util_2.readOnly)(Object, (0, util_1.createGetTrap)([
        'keys',
        'values',
        'hasOwnProperty',
        'fromEntries',
        'assign',
        'create',
    ])),
    Promise: (0, util_2.readOnly)(Promise, (0, util_1.createGetTrap)([
        'all',
        'race',
        'resolve',
        'reject',
        'allSettled',
    ])),
    Date: (0, util_2.readOnly)(Date, (0, util_1.createGetTrap)([
        'now',
        'parse',
        'UTC',
    ])),
    Array: (0, util_2.readOnly)(Array, (0, util_1.createGetTrap)([
        'isArray',
        'from',
        'of',
    ])),
    Number: (0, util_2.readOnly)(Number, (0, util_1.createGetTrap)([
        'isFinite',
        'isInteger',
        'isNaN',
        'isSafeInteger',
        'parseFloat',
        'parseInt',
        'MAX_VALUE',
        'MIN_VALUE',
        'NaN',
        'NEGATIVE_INFINITY',
        'POSITIVE_INFINITY',
        'MAX_SAFE_INTEGER',
        'MIN_SAFE_INTEGER',
        'EPSILON',
    ])),
    String: (0, util_2.readOnly)(String, (0, util_1.createGetTrap)([
        'fromCharCode',
        'fromCodePoint',
        'raw',
    ])),
    // errors
    Error: (0, util_2.readOnly)(Error, (0, util_1.createGetTrap)([])),
    EvalError: (0, util_2.readOnly)(EvalError, (0, util_1.createGetTrap)([])),
    RangeError: (0, util_2.readOnly)(RangeError, (0, util_1.createGetTrap)([])),
    ReferenceError: (0, util_2.readOnly)(ReferenceError, (0, util_1.createGetTrap)([])),
    SyntaxError: (0, util_2.readOnly)(SyntaxError, (0, util_1.createGetTrap)([])),
    TypeError: (0, util_2.readOnly)(TypeError, (0, util_1.createGetTrap)([])),
    URIError: (0, util_2.readOnly)(URIError, (0, util_1.createGetTrap)([])),
};
const readOnlyNativesNames = Object.keys(readOnlyNatives);
const createCfunc = (config = {}) => {
    const { timeout = defaultTimeout, memoryLimit = defaultMemoryLimit, source = '', filename = 'cfunc:file', } = config;
    let { globalNames = [] } = config;
    globalNames = Array.from(new Set([...readOnlyNativesNames, ...globalNames]));
    globalNames.forEach((name) => {
        if (!(0, util_1.isValidIdentifier)(name)) {
            throw new Error(`Invalid identifier '${name}'`);
        }
        if (reserverGlobalIdentifiers.indexOf(name) !== -1) {
            throw new Error(`Reserved identifier '${name}'`);
        }
    });
    const { code = '' } = (0, compiler_1.compile)(source, allowedNodeTypes, globalNames);
    const __globals = '__globals';
    const resetContext = (0, endent_1.default) `const {
    ${globalNames.join(',\n')}
  } = ${__globals};`;
    const transformedCode = (0, endent_1.default) `exports.default = (${__globals}) => {
    ${resetContext}
    ${__globals} = void 0;

    ${`return ${code}`}
  }`;
    const script = new vm_1.Script(transformedCode, {
        filename,
    });
    const vmCtx = {
        createRuntime: () => {
            return (0, runtime_1.createRuntime)({
                timeout,
                memoryLimit,
            });
        },
        exports: Object.create(null),
    };
    // evaluate js code
    script.runInNewContext(vmCtx, {
        displayErrors: true,
        timeout: timeout,
        breakOnSigint: true,
    });
    // get the exported function, and create the wrapper
    const fn = vmCtx.exports.default;
    const contained = (args, globals) => {
        const _globals = globals || {};
        return (fn
            .call(null, Object.assign(Object.assign({}, readOnlyNatives), _globals))
            .apply(null, args || []));
    };
    contained.source = transformedCode;
    return contained;
};
exports.createCfunc = createCfunc;
//# sourceMappingURL=cfunc.js.map