"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createRuntime = exports.Runtime = void 0;
const util_1 = require("../server/util");
const error_1 = require("../server/error");
const util_2 = require("./util");
var CheckType;
(function (CheckType) {
    CheckType[CheckType["Sync"] = 0] = "Sync";
    CheckType[CheckType["Async"] = 1] = "Async";
})(CheckType || (CheckType = {}));
class Runtime {
    constructor(config) {
        this.error = void 0;
        this.memorySize = 0;
        this.config = Object.assign({ timeout: (0, util_1.mins)(10), syncTimeout: 100, memoryLimit: 1 * 1024 * 1024 * 10 }, config);
        this.beginTimestamp = Date.now();
        this.lastAsyncCheckTimestamp = this.beginTimestamp;
        this.lastCheck = CheckType.Async;
        this.sizeCache = new WeakMap();
    }
    throwTimeoutError() {
        this.error = new error_1.TimeoutError('Timeout error');
        throw this.error;
    }
    throwMemoryLimitError() {
        this.error = new error_1.MemoryLimitError(`Memory limit error, max: ${(0, util_2.formatBytes)(this.config.memoryLimit)}, reached: ${(0, util_2.formatBytes)(this.memorySize)}`);
        throw this.error;
    }
    checkSync() {
        if (this.error)
            this.throwTimeoutError();
        const now = Date.now();
        const { syncTimeout, timeout } = this.config;
        if (this.lastCheck === CheckType.Sync
            && ((now - this.lastAsyncCheckTimestamp > syncTimeout) || (now - this.beginTimestamp > timeout))) {
            this.throwTimeoutError();
        }
        this.lastCheck = CheckType.Sync;
    }
    checkAsync() {
        if (this.error)
            this.throwTimeoutError();
        const now = Date.now();
        const { timeout } = this.config;
        if (now - this.beginTimestamp > timeout) {
            this.throwTimeoutError();
        }
        this.lastAsyncCheckTimestamp = now;
        this.lastCheck = CheckType.Async;
        return new Promise((resolve) => setImmediate(resolve));
    }
    // memory
    sizeOf(value) {
        if ((0, util_1.isObject)(value)) {
            let size = this.sizeCache.get(value);
            if (size === void 0) {
                size = (0, util_2.sizeOf)(value);
                this.sizeCache.set(value, size);
            }
            return size;
        }
        else {
            return (0, util_2.sizeOf)(value);
        }
    }
    alloc(newAlloc, oldAlloc = void 0, container = void 0) {
        const sizeDiff = this.sizeOf(newAlloc) - this.sizeOf(oldAlloc);
        if ((0, util_1.isObject)(container) && sizeDiff !== 0) {
            this.sizeCache.set(container, this.sizeOf(container) + sizeDiff);
        }
        this.memorySize = this.memorySize + sizeDiff;
        if (this.memorySize > this.config.memoryLimit) {
            this.throwMemoryLimitError();
        }
    }
    captureLazyValue(value) {
        if ((0, util_1.isFunction)(value)) {
            return (...args) => this.captureLazyValue(value(...args));
        }
        else if ((0, util_1.isThenable)(value)) {
            return value.then((result) => this.captureLazyValue(result));
        }
        else {
            this.alloc(value);
            return value;
        }
    }
    createProxy(obj) {
        this.alloc(obj);
        return new Proxy(obj, {
            set: (obj, prop, value, receiver) => {
                this.alloc(value, obj[prop], obj);
                return Reflect.set(obj, prop, value, receiver);
            },
            deleteProperty: (obj, prop) => {
                this.alloc(void 0, obj[prop], obj);
                delete obj[prop];
                return true;
            }
        });
    }
    createArr(arr) {
        return this.createProxy(arr);
    }
    createObj(obj) {
        return this.createProxy(obj);
    }
    getProp(obj, prop) {
        let val = (0, util_2.isProtectedProperty)(prop) ? void 0 : obj[prop];
        if ((0, util_1.isFunction)(val)) {
            val = obj[prop].bind(obj);
        }
        return this.captureLazyValue(val);
    }
    callProp(obj, prop, ...args) {
        return (0, util_2.isProtectedProperty)(prop) ? void 0 : obj[prop](...args);
    }
    setProp(obj, prop, value, operator) {
        if ((0, util_1.isFunction)(value)) {
            throw new Error('Object does not accept function');
        }
        return (0, util_2.isProtectedProperty)(prop) ? void 0 : obj[prop] = value;
    }
    computedProp(prop) {
        return (0, util_2.isProtectedProperty)(prop) ? void 0 : prop;
    }
    async awaitPromise(promise) {
        const result = await promise;
        // do not await chackAsync because it is resolving a promise
        // so, it does not overtakes
        this.checkAsync();
        return result;
    }
}
exports.Runtime = Runtime;
const createRuntime = (config) => {
    return new Runtime(config);
};
exports.createRuntime = createRuntime;
//# sourceMappingURL=runtime.js.map