"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createClient = exports.Client = void 0;
var p_defer_1 = __importDefault(require("p-defer"));
var util_1 = require("./util");
var MAX_BATCH_SIZE_LIMIT = 1000;
/** Generic client */
var Client = /** @class */ (function () {
    function Client(config) {
        if (config === void 0) { config = {}; }
        if (!config.handler) {
            throw new Error('invalid handler');
        }
        this.config = {};
        this.setConfig(config);
        this.isUsingBatch = false;
        this.batchedRequests = [];
        this.batchedRequestsDeferredPromises = [];
        this.requestPromiseDedupeMap = new Map();
    }
    Client.prototype.setConfig = function (config) {
        this.config = __assign({ deduplicate: true, response: util_1.noop }, config);
    };
    Client.prototype.unscheduleBatch = function () {
        clearTimeout(this.batchScheduleTimeout);
        this.batchScheduleTimeout = void 0;
    };
    Client.prototype.scheduleBatch = function (timeout) {
        this.unscheduleBatch();
        var _timeout = Number(timeout);
        var schedule = _timeout >= 0 ? function (fn) { return setTimeout(fn, _timeout); } : function () { return Infinity; };
        this.batchScheduleTimeout = schedule(this.flush.bind(this));
    };
    /** Execute a client request */
    Client.prototype.request = function (source, args, sourceLoc) {
        var _this = this;
        var _a, _b;
        var deduplicate = this.config.deduplicate;
        var idx = this.batchedRequests.length;
        var deferredPromise = (0, p_defer_1.default)();
        var deduplicateKey = source + JSON.stringify(args);
        var existingPromise = this.requestPromiseDedupeMap.get(deduplicateKey);
        var request = {
            index: idx,
            source: source,
            args: args,
            sourceLoc: sourceLoc,
        };
        if (deduplicate && existingPromise) {
            existingPromise.then(deferredPromise.resolve);
            existingPromise.catch(deferredPromise.reject);
        }
        else {
            this.batchedRequests[idx] = request;
            this.batchedRequestsDeferredPromises[idx] = deferredPromise;
            this.requestPromiseDedupeMap.set(deduplicateKey, deferredPromise.promise);
        }
        if (this.isUsingBatch) {
            var sizeLimit = Math.min(Number((_a = this.batchConfig) === null || _a === void 0 ? void 0 : _a.sizeLimit), MAX_BATCH_SIZE_LIMIT);
            if (this.batchedRequests.length >= sizeLimit) {
                this.flush();
            }
            else if (this.batchScheduleTimeout === void 0) {
                this.scheduleBatch((_b = this.batchConfig) === null || _b === void 0 ? void 0 : _b.timeout);
            }
        }
        else {
            this.flush();
        }
        return deferredPromise.promise.then(function (result) {
            _this.requestPromiseDedupeMap.delete(deduplicateKey);
            return result;
        }, function (reason) {
            _this.requestPromiseDedupeMap.delete(deduplicateKey);
            throw reason;
        });
    };
    /** Flush batched requests */
    Client.prototype.flush = function () {
        var _this = this;
        this.unscheduleBatch();
        if (this.batchedRequests.length === 0)
            return;
        var handler = this.config.handler;
        var requests = this.batchedRequests;
        var deferredPromises = this.batchedRequestsDeferredPromises;
        // reset
        this.batchedRequests = [];
        this.batchedRequestsDeferredPromises = [];
        var write = function (resp) {
            var _a, _b, _c;
            var index = resp.index;
            var deferredPromise = deferredPromises[index];
            _this.config.response(resp);
            if (resp && resp.error === void 0) {
                deferredPromise.resolve(resp.result);
            }
            else {
                var error = new Error((_a = resp.error) === null || _a === void 0 ? void 0 : _a.message);
                error.name = (_c = (_b = resp.error) === null || _b === void 0 ? void 0 : _b.name) !== null && _c !== void 0 ? _c : 'Error';
                deferredPromise.reject(error);
            }
        };
        var end = function (reason) {
            deferredPromises.forEach(function (d) { return d.reject(reason); });
        };
        handler({
            getRequests: function () {
                return requests;
            },
            write: write,
            end: end,
        });
    };
    /** Turn batch mode on or off */
    Client.prototype.useBatch = function (config) {
        this.flush();
        this.isUsingBatch = !!config;
        config = typeof config === 'boolean' ? {} : config;
        this.batchConfig = __assign({ timeout: -1, sizeLimit: MAX_BATCH_SIZE_LIMIT }, config);
    };
    return Client;
}());
exports.Client = Client;
/** Create client instance */
var createClient = function (config) {
    return new Client(config);
};
exports.createClient = createClient;
//# sourceMappingURL=client.js.map