"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.extractDts = void 0;
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const globby_1 = __importDefault(require("globby"));
const make_dir_1 = __importDefault(require("make-dir"));
const rimraf_1 = __importDefault(require("rimraf"));
const extractDts = async (apiModulePath, destinationDir, tscConfig = {}) => {
    const sourcePath = path_1.default.resolve(apiModulePath);
    destinationDir = path_1.default.resolve(destinationDir);
    const signatureFilePath = path_1.default.join(destinationDir, '/.extract-dts');
    if (fs_1.default.existsSync(signatureFilePath)) {
        rimraf_1.default.sync(destinationDir);
    }
    const dtsDir = await (0, make_dir_1.default)(path_1.default.join(destinationDir, '/dts'));
    fs_1.default.writeFileSync(signatureFilePath, '');
    const fileMapping = compile([sourcePath], Object.assign({ target: 'ES2019', lib: ['lib.es2019.full.d.ts'], strict: false, declaration: true, skipLibCheck: true, allowJs: true, emitDeclarationOnly: true, noEmitOnError: false, declarationDir: dtsDir, esModuleInterop: true }, tscConfig));
    const mainFileDestination = fileMapping[sourcePath];
    fs_1.default.writeFileSync(path_1.default.join(destinationDir, '/index.d.ts'), `export * from './${path_1.default.relative(destinationDir, mainFileDestination.slice(0, mainFileDestination.length - '.d.ts'.length))}';`);
    const dtsPaths = await (0, globby_1.default)([path_1.default.join(destinationDir, '/**/*.d.ts')]);
    dtsPaths.forEach(file => {
        const filename = path_1.default.join(path_1.default.dirname(file), path_1.default.basename(file, '.d.ts')) + '.js';
        if (!fs_1.default.existsSync(filename)) {
            fs_1.default.writeFileSync(filename, '');
        }
    });
};
exports.extractDts = extractDts;
const compile = (fileNames, options, _log = true) => {
    let ts;
    try {
        ts = require('typescript');
    }
    catch (_) {
        throw new Error('Please add TypeScript as dependency');
    }
    const log = _log ? console.log.bind(console) : (...args) => { };
    const host = ts.createCompilerHost(options);
    const hostGetSourceFile = host.getSourceFile.bind(host);
    const hostWriteFile = host.writeFile.bind(host);
    const fileMapping = {};
    const getDisplayPath = (fileName) => {
        const currentDirectory = host.getCurrentDirectory() + path_1.default.sep;
        let displayFileName = fileName;
        if (fileName.indexOf(currentDirectory) === 0) {
            displayFileName = fileName.slice(currentDirectory.length);
        }
        return displayFileName;
    };
    log(`\nImporting from: ${host.getCurrentDirectory()}\n`);
    log(`Sources:`);
    host.getSourceFile = (fileName, ...restArgs) => {
        log(`${getDisplayPath(fileName)}`);
        return hostGetSourceFile(fileName, ...restArgs);
    };
    host.writeFile = (...args) => {
        const [fileName, _, __, ___, sourceFiles] = args;
        const sourceFile = sourceFiles[0];
        if (sourceFile) {
            fileMapping[sourceFile.fileName] = fileName;
            log(`${getDisplayPath(fileName)}`);
        }
        return hostWriteFile(...args);
    };
    const program = ts.createProgram(fileNames, options, host);
    log(`\nCompiled:`);
    const emitResult = program.emit();
    const allDiagnostics = ts
        .getPreEmitDiagnostics(program)
        .concat(emitResult.diagnostics);
    allDiagnostics.forEach((diagnostic) => {
        if (diagnostic.file) {
            const { line, character } = diagnostic.file.getLineAndCharacterOfPosition(diagnostic.start);
            const message = ts.flattenDiagnosticMessageText(diagnostic.messageText, '\n');
            log(`${diagnostic.file.fileName} (${line + 1},${character + 1}): ${message}`);
        }
        else {
            log(`${ts.flattenDiagnosticMessageText(diagnostic.messageText, '\n')}`);
        }
    });
    return fileMapping;
};
//# sourceMappingURL=extract-dts.js.map