"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createEngine = exports.Engine = void 0;
const util_1 = require("./util");
const error_1 = require("./error");
const service_1 = require("./service");
const cache_1 = require("./cache");
const cfunc_1 = require("../cfunc");
const rpc_1 = require("./rpc");
const partial_func_1 = require("./partial-func");
/** Run JavaScript query functions */
class Engine {
    constructor(config) {
        this.config = Object.assign({ timeout: (0, util_1.mins)(1), displayErrors: true }, config);
        this.readSource = this.config.source || util_1.identity;
        this.cfuncCache = new cache_1.Cache();
        this.services = config.services || {};
        this.servicesKeys = Object.keys(this.services);
        const servicesPath = this.config.servicesPath;
        if (typeof servicesPath === 'string' && servicesPath !== '') {
            if (config.services) {
                throw new Error(`Can provide either 'servicesPath' or 'services'`);
            }
            this.servicesModule = Promise.resolve().then(() => __importStar(require(servicesPath))).then((module) => {
                this.services = module;
                this.servicesKeys = Object.keys(this.services);
                this.servicesModule = void 0;
            });
        }
    }
    /** Return engine configuration */
    getConfig() {
        return Object.assign({}, this.config);
    }
    /** Run query function */
    async run(source, args, queryContext) {
        if (this.servicesModule)
            await this.servicesModule;
        const createContext = (0, util_1.isFunction)(this.config.context) ? this.config.context : util_1.noop;
        const ctx = await createContext(queryContext);
        const src = await this.readSource(source);
        return await this.execute(src, args, ctx, queryContext);
    }
    async execute(source, args, serviceContext, queryContext) {
        const onError = this.config.displayErrors ? console.error.bind(console) : util_1.noop;
        const contextifiedServices = (0, service_1.instantiateServices)(this.services, serviceContext);
        const globals = Object.assign({}, contextifiedServices);
        if ((0, rpc_1.isRpcCommand)(source)) {
            return this.handleRpc((0, rpc_1.parseRpcCommand)(source), args, contextifiedServices, onError);
        }
        const parsedFunc = this.parseFunc(source, onError, globals, queryContext);
        const _args = await Promise.all((args || []).map(async (arg) => {
            if ((0, util_1.isString)(arg) && (0, partial_func_1.isPartialFunc)(arg)) {
                // read source
                const partialFunctionSource = await this.readSource((0, partial_func_1.getPartialFuncSource)(arg));
                const parsedPartialFunc = this.parseFunc(partialFunctionSource, onError, globals);
                return (...args) => parsedPartialFunc(args);
            }
            return arg;
        }));
        return parsedFunc(_args);
    }
    parseFunc(source, onError, globals, queryContext) {
        let cfunc = this.cfuncCache.get(source);
        if (!cfunc) {
            try {
                cfunc = (0, cfunc_1.createCfunc)({
                    globalNames: this.servicesKeys,
                    timeout: this.config.timeout,
                    source,
                });
                this.cfuncCache.set(source, cfunc);
            }
            catch (err) {
                let location = '';
                if (queryContext && queryContext.sourceLoc) {
                    const sourceLoc = queryContext.sourceLoc;
                    location = `Source location: ${sourceLoc}`;
                }
                const error = new error_1.EvalError(`in query: ${location} \n ${source} \n ${String(err)}`);
                onError(error);
                throw error;
            }
        }
        return (args) => {
            return cfunc(args, globals).catch((err) => {
                onError(new Error(`in query query: ${source} \n ${String(err.stack)}`));
                throw err;
            });
        };
    }
    handleRpc(rpcCommand, args, contextifiedServices, onError) {
        // handle rpc
        const { service, method } = rpcCommand;
        if (this.servicesKeys.indexOf(service) === -1) {
            const error = new error_1.EvalError(`invalid service name: ${service}}`);
            onError(error);
            throw error;
        }
        return contextifiedServices[service][method].apply(null, args);
    }
    /** Return path of registered endpoints */
    getEndpointPaths() {
        const paths = [];
        const services = (0, service_1.instantiateServices)(this.services);
        Object.keys(services).forEach(serviceName => {
            const service = services[serviceName];
            Object.keys(service).forEach(endpointName => {
                paths.push(`${serviceName}.${endpointName}`);
            });
        });
        return paths;
    }
}
exports.Engine = Engine;
/** Create new Engine */
const createEngine = (config = {}) => {
    return new Engine(config);
};
exports.createEngine = createEngine;
//# sourceMappingURL=engine.js.map