"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.handleHttpRequest = void 0;
const p_settle_1 = __importDefault(require("p-settle"));
const engine_1 = require("./engine");
const error_1 = require("./error");
const json_stream_1 = require("../client/json-stream");
/** Map errors to HTTP status */
const getHttpStatusFromError = (err) => {
    switch (err.errorType) {
        case error_1.ErrorType.TIMEOUT: return 408;
        case error_1.ErrorType.EVAL: return 400;
        case error_1.ErrorType.RUNTIME: return 500;
        default: return 500;
    }
};
/** Generic HTTP handler, adaptable by providing handler interface */
const handleHttpRequest = async (iface) => {
    try {
        const { initialContext = {}, engine = (0, engine_1.createEngine)() } = iface;
        // Supports-Web-Streams is a custom header to let the server know if it can
        // stream back the response
        const supportsWebStreams = Number(iface.getHeader('Supports-Web-Streams'));
        // status code 207 (Multi-Status) because the HTTP response is a batch that can contain
        // successful of failed responses packages
        iface.setStatusCode(207);
        iface.setHeader('Content-Type', 'text/plain;charset=utf-8');
        if (supportsWebStreams === 1) {
            // prepare the response to be streamed
            iface.setHeader('Transfer-Encoding', 'chunked');
        }
        let requests;
        switch (String(iface.getMethod()).toUpperCase()) {
            case 'GET':
                requests = iface.getQueryParam('requests');
                break;
            case 'POST':
                requests = iface.getBody();
                break;
            default:
                throw new Error('method not allowed');
        }
        const resultPromises = [];
        // create JSON stream strigifier and bind to the write function of the interface
        const strigifier = (0, json_stream_1.createStringifier)({ onData: iface.write });
        // init JSON stream parser
        const parser = (0, json_stream_1.createParser)({
            onData(data) {
                const { index, source, args, sourceLoc } = data;
                const ctx = Object.assign(Object.assign({}, initialContext), { source, args, sourceLoc });
                const resultPromise = engine.run(source || '', args, ctx).then(result => {
                    strigifier.write({ index, result, sourceLoc });
                }).catch((err = {}) => {
                    const name = err.name;
                    const message = err.message;
                    strigifier.write({ index, error: { name, message }, sourceLoc });
                });
                resultPromises.push(resultPromise);
            }
        });
        parser.write(requests);
        strigifier.close();
        parser.close();
        // wait for all engine run results
        await (0, p_settle_1.default)(resultPromises);
    }
    catch (err) {
        iface.setStatusCode(getHttpStatusFromError(err));
        iface.write(err.stack || err);
    }
    finally {
        iface.end();
    }
};
exports.handleHttpRequest = handleHttpRequest;
//# sourceMappingURL=http.js.map